unit Serial; {pro Turbo Pascal 7.0 - verze 5/99b}

{ ZKLADN podpora pro komunikaci po sriov lince (RS-232) na rozumn
  (tj. hardwarov) rovni za pouit peruen a bufferu.

  Umouje: - prci s jednm portem (COM1 a COM4) s monost on-line zmny
            - nastaven IRQ v ppad poteby (vetn > 7)
            - nastaven velikosti vstupn bufferu
            - testovat stav linky a modemu (pokud tomu nkdo rozum)

  Chyb: - zapnn vstupn FIFO u 16550 (snad nkdy bude)
         - jakkoli kontrola smysluplnosti pedvanch parametr
         - oeten rznch neobvyklch stav, kter nemaj nic spolenho
           se slunm bnm provozem :-)
         - podpora vce ne 4 port, drivery pro Windows NT a mrkev :-))

  Pehled zmn:  5/99 (opravdu:-) opravena prce s IRQ > 7
                 9/98 zmna ve vbru adresy portu - nen-li adresa uvedena
                      v datov oblasti BIOSu, pouije se standardn hodnota
                 6/97 doplnna monost vysln "naslepo"
                11/96 zkladn verze sepsan na zklad podobnho unitu
                      jmnem ComDrv, staenho z CeMENT BBS Praha

Ŀ
 UPOZORNN: Tento soubor je PUBLIC DOMAIN, take si s nm mete dlat, 
 co chcete (teba vytisknout a povsit na ze:-), ale NA VLASTN RIZIKO. 


                                  Michal Jungmann <jungmann@braillnet.cz> }

interface

const pNone = 0; {konstanty pro nastaven parity}
      pOdd = 1;
      pEven = 3;

const comIrq: array[1..4] of Byte = (4, 3, 4, 3); {peruen pro bn COMy}
                                            {mono zmnit pro intern modem}

const comAddr: array[1..4] of Word = ($3F8, $2F8, $3E8, $2E8);
                                                   {standardn adresy port}
                 {upravuj se pi startu podle daj BIOSu na 0:0400-0:0407}

var overRun: Boolean; {TRUE pokud nkdy petekl buffer - mono testovat}

const sendBlind: Boolean = FALSE; {pokud nastavte TRUE, nebude SendChar
                                   testovat, zda ji me poslat}

function GetOk: Boolean; {je nco v bufferu?}
function GetChar: Char; {vyber znak z bufferu; ekej, pokud nen}

function SendOk: Boolean; {mu poslat?}
procedure SendChar(c: Char); {poli znak; ekej, ne to pjde}

procedure ClearBuffer; {vyma buffer}

procedure Init(comNo, wordLen, stopBits, parity: Byte; baudRate: LongInt;
               buffLen: Word); {nastav parametry, peruen, buffer ap.}
               {slo portu (1 a 4), dlka slova (8), stop bity (1),
               parita (pNone), baudRate (dle uven), dlka bufferu}
               {POZOR! Nen testovna smysluplnost poslanch hodnot!!!}

procedure Done; {odinstaluje peruen, zru buffer...}
                {standardn probh automaticky pi skonen}

function LineStatus: Byte; {viz SERIE.TXT}
function ModemStatus: Byte; {tamt}


implementation
uses Dos;

type TBuffer = array[0..65520] of Byte;
     PBuffer = ^TBuffer;

const buffer: PBuffer = nil;

var intNo, newMask21, newMaskA1, oldMask21, oldMaskA1: Byte;
    portAddr, index, buffTop, buffLength: Word;
    oldInt, oldExit: Pointer;

function LineStatus: Byte; assembler;
asm
 mov DX, portAddr
 add DX, 0005
 in AL, DX
end; {LineStatus}

function ModemStatus: Byte; assembler;
asm
 mov DX, portAddr
 add DX, 0006
 in AL, DX
end; {ModemStatus}

function GetOk: Boolean;
begin
 GetOk:= index <> buffTop;
end; {GetOk}

function GetChar: Char;
begin
 repeat until GetOk;
 GetChar:= Char(buffer^[index]);
 index:= (index + 1) mod buffLength;
end; {GetChar}

function SendOk: Boolean;
begin
 SendOk:= LineStatus and $20 > 0; {bit 5}
end; {SendOk}

procedure SendChar(c: Char);
begin
 repeat until sendBlind or SendOk;
 port[portAddr]:= Byte(c);
end; {SendChar}

procedure ClearBuffer;
begin
 index:= 0;
 buffTop:= 0;
 overRun:= False;
end; {ClearBuffer}

procedure IrqOn;
begin
 asm cli end;
 Port[$21]:= newMask21 and Port[$21];
 Port[$A1]:= newMaskA1 and Port[$A1];
 asm sti end;
end; {IrqOn}

procedure IrqOff;
begin
 asm cli end;
 Port[$21] := not newMask21 or Port[$21];
 Port[$A1] := not newMaskA1 or Port[$A1];
 asm sti end;
end; {IrqOff}

{$F+}
procedure SaveChar; interrupt;
begin
 asm
  mov AL, $20
  out $A0, AL
  out $20, AL
  sti
 end; {asm}
 buffer^[buffTop]:= port[portAddr];
 buffTop:= (buffTop + 1) mod buffLength;
 overRun:= overRun or (buffTop = index);
end; {SaveChar}

procedure Done;
begin
 if buffer <> nil then
  begin
   port[PortAddr+1]:= 0; {disable interrupts}
   IrqOff;
   SetIntVec(intNo, oldInt);
   port[$21]:= oldMask21;
   port[$A1]:= oldMaskA1;
   ExitProc:= oldExit;
   FreeMem(buffer, buffLength);
   buffer:= nil;
  end;
end; {Done}
{$F-}

procedure Init;
const maxBaudRate = 115200;
var w: Word;
begin
 if buffer = nil then
  begin
   if MaxAvail > buffLen then buffLength:= buffLen
   else buffLength:= MaxAvail;
   GetMem(buffer, buffLength);
   ClearBuffer;

   portAddr:= comAddr[comNo];

   oldMask21:= port[$21];
   oldMaskA1:= port[$A1];
   if comIrq[comNo] < 8  then
    begin
     intNo:= comIrq[comNo] + 8;
     newMask21:= Byte(not (1 shl comIrq[comNo]));
     newMaskA1:= 255; {ve zakzno}
    end
   else
    begin
     intNo:= comIrq[comNo] + $68;
     newMask21:= 251; {povoleno IRQ2 - kaskda}
     newMaskA1:= Byte(not (1 shl (comIrq[comNo] - 8)));
    end;

   w:= port[portAddr];
   w:= port[portAddr+5];
   w:= maxBaudRate div baudRate;
   port[portAddr+3]:= $80; {DLAB = 1}
   port[PortAddr+1]:= Hi(w);
   port[PortAddr  ]:= Lo(w);
   port[PortAddr+3]:= parity shl 3 or (stopBits - 1) shl 2 or (wordLen - 5);

   port[PortAddr+1]:= 0; {disable interrupts}
   IrqOff;
   port[PortAddr+4]:= $0F; {DTR,RTS,OUT1,OUT2}

   GetIntVec(intNo, oldInt);
   SetIntVec(intNo, @SaveChar);

   w:= port[PortAddr];
   w:= port[PortAddr+5];
   port[PortAddr+1]:= 1; {enable RCD interrupt}
   IrqOn;

   oldExit:= ExitProc;
   ExitProc:= @Done;
  end;
end; {Init}

procedure InitAddr;
var BiosAddr: array[1..4] of Word absolute 0:$400;
    i: Byte;
begin
 for i:= 1 to 4 do
  if BiosAddr[i] > 0 then comAddr[i]:= BiosAddr[i];
end; {InitAddr}

begin
 InitAddr;
end. {of unit}